#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include "contFromEdges.h"
/*#include "edges.h"
#include "globals.h"*/

int Num_Orientations=4;
double Sigma_c=1.3, Sigma_s=1.3;
/*int O_Contrast=1, I_Contrast=0;*/
int Radius=3;
/*int O_Target=180;*/
int CS_Ratio=3;


main(int argc, char *argv[])
{
  double **gaus_c, **gaus_s;
  int level_size, max;
  unsigned char **edge_data, *char_con_map;
  double **feat_con_map, *con_map, *norm_con_map;
  int i;

  if (argc==1)
  {
    printf("\nExecutable: x_contFromEdges\n");
    printf("Input: edge maps\n");
    printf("Output:  contrast map of same dimension as edge maps\n");
    exit(0);
  }

  Num_Orientations=argc-1;

  /* Allocate storage for Gaussians */
  gaus_c=(double **) calloc(2*Radius+1, sizeof(double*));
  for (i=0; i<2*Radius+1; i++)
    gaus_c[i]=(double *) calloc(2*Radius+1, sizeof(double));

  gaus_s=(double **) calloc(2*(3*Radius+1)+1, sizeof(double*));
  for (i=0; i<2*(3*Radius+1)+1; i++)
    gaus_s[i]=(double *) calloc(2*(3*Radius+1)+1, sizeof(double));

  /* Allocate storage for correct number of edge maps */
  edge_data=(unsigned char**) calloc(Num_Orientations, sizeof(unsigned char*));

  /* Read edge data from files */
  for (i=0;  i<Num_Orientations; i++)
    max = read_pgm(argv[i+1], &edge_data[i], &level_size, &level_size);

  /* Allocate storage for contrast maps */
  norm_con_map = (double *) calloc(level_size*level_size, sizeof(double));
  con_map = (double *) calloc(level_size*level_size, sizeof(double));
  char_con_map = (unsigned char *) calloc(level_size*level_size, 1);

  /* Calculate contrast */
  max=contrast(edge_data,
                 &feat_con_map,
                 gaus_c, gaus_s,
                 level_size, Radius, Num_Orientations);

  combine_maps(max, con_map, feat_con_map, level_size*level_size);

  /* normalize contrast map */
  norm_dbl(con_map, norm_con_map, level_size*level_size, max);

  /* convert to unsigned char */
  d_to_c(norm_con_map, char_con_map, level_size*level_size);

  /* write as pgm image */
  write_pgm("hope.pgm", char_con_map, level_size, level_size, 255);

}

void norm_dbl(double *from, double *to, int size, double max)
{
  int i;
  double factor=255.0/max;
  FILE *fa, *fb;

  for(i=0; i<size; i++)
    to[i] = factor>0.0 ? from[i]*factor : 0.0;
}

void d_to_c(double *from, unsigned char *to, int size)
{
  int i;
  for (i=0; i<size; i++)
    to[i] = from[i]>0.0 ? (unsigned char)from[i] :0;
}


double contrast(unsigned char **feature_map,  
		double **(*dbl_contrast),
		double **gaus_c, double **gaus_s,
		int width, 
		int radius_c, 
		int num_features)
{
  int size = width * width;
  int radius_s=radius_c*CS_Ratio+1;
  unsigned char *char_map;
  double max_inten;
  int i;
  double max;

  static int level=0;
  char f_name[20];
  unsigned char *temp_c;
  double *temp_d;

  (*dbl_contrast) = (double **) calloc(num_features,  sizeof(double *));
  for (i=0; i<num_features; i++)
  {
    (*dbl_contrast)[i] = (double *) calloc(size,  sizeof(double));
  }
  
  /* calculate values for gaussians */
  Calc_Gaus(gaus_c, radius_c, radius_c*Sigma_c);
  Calc_Gaus(gaus_s, radius_s, radius_s*Sigma_s);


  max=Feature_Contrast(width, radius_c, radius_s, gaus_c, gaus_s,
			feature_map, (*dbl_contrast), num_features);

  return max;

} /* contrast()*/


void combine_maps(double max, double *values, double **o_contrast, int size)
{
  int i, j;
  double tmax, factor;
  char f_name[20];
  double **contrast_feature;
  int start, end;
  int o_start=0, o_end;

  factor=255.0/max;

  printf("max=%g\n", max);


  for (i=0; i<size; i++)
  {
    tmax=0;

    for (j=0; j<Num_Orientations; j++)
      if (o_contrast[j][i] > tmax)
	tmax=o_contrast[j][i];

    values[i] = (double) (tmax*factor);
  } /* for (i) */

} /* combine_maps */


double Feature_Contrast(int width, int center_radius, int radius, 
		   double **gaus_c, double **gaus_s,
		   unsigned char **edges, double **contrast_map,
		   int num_features)
{
  int i, j, pixel_x, pixel_y;
  int temp;
  float weight[num_features];
  double cell[num_features];
  double max=0.0;

  Calc_Weights(weight, num_features);

  for (pixel_y=radius; pixel_y<width-radius; pixel_y++)
    for (pixel_x=radius; pixel_x<width-radius; pixel_x++)
    {
      for(i=0; i<num_features; i++)
      {
        cell[i]=WEIGHT*Convolve(pixel_x, pixel_y, width,
                         center_radius, gaus_c, edges[i]);
        cell[i]=cell[i]-Convolve(pixel_x, pixel_y, width, radius,
                                 gaus_s, edges[i]);
      }

      for(i=0; i<num_features; i++)
      {
        for(j=0; j<num_features; j++)
	{
          contrast_map[i][pixel_y*width+pixel_x] 
			+= weight[(j-i+num_features)%num_features]
			   * cell[j];
	}
        if (contrast_map[i][pixel_y*width+pixel_x]>max)
	  max=contrast_map[i][pixel_y*width+pixel_x];
      } /* for (i) */

    }/* for (pixels) */

  return max;

}/* Feature_Contrast()*/


void Calc_Gaus(double **gaus, int rad, double sigma)
{
  int i,j;
  double total=0.0;

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
    {
      gaus[i+rad][j+rad]=4*exp(-(float)(i*i+j*j)/(2.0*sigma));
      total+=gaus[i+rad][j+rad];
    }

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
      gaus[i+rad][j+rad]=gaus[i+rad][j+rad]/total;
}/* Calc_Gaus() */

void Calc_Weights(float *w, int num_features)
{
  int i;
  float denom;


  w[0]=1.0;
  w[1]=-0.1;
  w[2]=-0.8;
  w[3]=-0.1;
/*
  w[0]=1.0;
  w[1]=1.0;

  denom=num_features*num_features;
  if (num_features%2)
    denom=denom-1;
 

  for (i=1; i<=num_features/2; i++)
  {
    w[i]=(float)(-4*i)/denom;
    w[num_features-i]=w[i];
  }
*/
} /* Calc_Weights() */


double Convolve(int x, int y, int width, int rad, double **gaus,
                unsigned char *values)
{
  int i,j;
  double temp=0;

  for(i=-rad; i<=rad; i++)
    for(j=-rad; j<=rad; j++)
        temp+=(double) values[(y-i)*width+(x-j)] *
                gaus[i+rad][j+rad];

  return temp;
} /* Convolve */














